(ns jable.client-test
  (:require [clojure.test :refer :all]
            [jable.client :refer :all]))

(deftest write-command-test
  (testing "formatting command with no param"
    (let [buf (java.io.StringWriter.)]
      (write-command buf {:cmd "AWAY" :params []})
      (is (= (.toString buf) "AWAY\r\n"))))

  (testing "formatting command with one param"
    (let [buf (java.io.StringWriter.)]
      (write-command buf {:cmd "NICK" :params ["bar"]})
      (is (= (.toString buf) "NICK :bar\r\n"))))

  (testing "formatting command with two params"
    (let [buf (java.io.StringWriter.)]
      (write-command buf {:cmd "PRIVMSG" :params ["#chan" "hello"]})
      (is (= (.toString buf) "PRIVMSG #chan :hello\r\n"))))

  (testing "formatting command with three params"
    (let [buf (java.io.StringWriter.)]
      (write-command buf {:cmd "KICK" :params ["#chan" "badperson" "bye bye"]})
      (is (= (.toString buf) "KICK #chan badperson :bye bye\r\n")))))

(deftest parse-command-test
  (testing "parsing a command with no param"
    (is (= (parse-command "AWAY")
           {:tags {} :source nil :cmd "AWAY" :params []})))

  (testing "parsing a command with one param"
    (is (= (parse-command "AWAY reason")
           {:tags {} :source nil :cmd "AWAY" :params ["reason"]})))

  (testing "parsing a command with one trailing param"
    (is (= (parse-command "AWAY :the reason")
           {:tags {} :source nil :cmd "AWAY" :params ["the reason"]})))

  (testing "parsing a command with two params"
    (is (= (parse-command "PRIVMSG #chan :hi")
           {:tags {} :source nil :cmd "PRIVMSG" :params ["#chan" "hi"]})))

  (testing "parsing a command with one normal param and one trailing param"
    (is (= (parse-command "PRIVMSG #chan :hi there")
           {:tags {} :source nil :cmd "PRIVMSG" :params ["#chan" "hi there"]})))

  (testing "parsing a command with source"
    (is (= (parse-command ":nick!user@host AWAY")
           {:tags {} :source "nick!user@host" :cmd "AWAY" :params []})))

  (testing "parsing a command with a tag"
    (is (= (parse-command "@label=abc ACK")
           {:tags {"label" "abc"} :source nil :cmd "ACK" :params []})))

  (testing "parsing a command with tags"
    (is (= (parse-command "@label=abc;time=123 ACK")
           {:tags {"label" "abc", "time", "123"} :source nil :cmd "ACK" :params []})))

  (testing "parsing a command with source and tag"
    (is (= (parse-command "@label=abc :nick!user@host AWAY")
           {:tags {"label" "abc"} :source "nick!user@host" :cmd "AWAY" :params []})))

  (testing "parsing a command with all fields"
    (is (= (parse-command "@label=abc;time=123 :server. KICK #chan badperson :reason")
           {:tags {"label" "abc", "time", "123"}  :source "server." :cmd "KICK" :params ["#chan" "badperson" "reason"]}))))
